#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains utils functions for tpRigToolkit
"""

from __future__ import print_function, division, absolute_import

import os

import tpDcc as tp
from tpDcc.libs.python import folder, fileio, version, path as path_utils

import tpRigToolkit


def get_data_files_directory():
    """
    Returns default paths where data files for tpRigToolkit are located
    :return: list(str)
    """

    data_dirs = [os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'data')]

    # TODO: Implement a generic way to include data for specific DCCs without add them here explicitly
    if tp.is_maya():
        from tpRigToolkit.dccs.maya import data
        data_dirs.append(os.path.join(os.path.dirname(data.__path__[0]), 'data'))

    return data_dirs


def copy(source, target, description=''):
    """
    Copies given file or files and creates a new version of the file with the given description
    :param source: str, source file or folder we want to copy
    :param target: str, destination file or folder we want to copy into
    :param description: str, description of the new version
    """

    is_source_a_file = path_utils.is_file(source)

    if is_source_a_file:
        copied_path = fileio.copy_file(source, target)
    else:
        if not path_utils.exists(source):
            tpRigToolkit.logger.info('Nothing to copy: {}\t\tData was probably created but not saved yet.'.format(
                path_utils.get_dirname(is_source_a_file)))
            return
        if path_utils.exists(target):
            folder.delete_folder(target)
        copied_path = folder.copy_folder(source, target)

    if not copied_path:
        tpRigToolkit.logger.warning('Error copying {}\t to\t{}'.format(source, target))
        return

    if copied_path > -1:
        tpRigToolkit.logger.info('Finished copying {} from {} to {}'.format(description, source, target))
        version_file = version.VersionFile(copied_path)
        version_file.save('Copied from {}'.format(source))



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains core data widgets for tpRigToolkit
"""

from __future__ import print_function, division, absolute_import

import os

from Qt.QtCore import *
from Qt.QtWidgets import *

from tpDcc.libs.python import settings, path as path_utils
from tpDcc.libs.qt.core import qtutils
from tpDcc.libs.qt.widgets.library import manager, items, loadwidget

import tpRigToolkit


class DataFolder(manager.LibraryDataFolder, object):
    def __init__(self, name, file_path, data_path=None):
        super(DataFolder, self).__init__(name=name, file_path=file_path, data_path=data_path)

    def get_manager(self):
        """
        Implements base manager.LibraryDataFolder get_manager function
        We use tpRigToolkit data manager
        :return: LibraryManager
        """

        return tpRigToolkit.DataMgr()


class DataPreviewWidget(loadwidget.LoadWidget, object):
    def __init__(self, item, parent=None):
        super(DataPreviewWidget, self).__init__(item=item, parent=parent)

    def ui(self):
        super(DataPreviewWidget, self).ui()

        self._export_btn = QPushButton('Export')
        self._export_btn.setObjectName('exportButton')
        self._export_btn.setMinimumSize(QSize(60, 35))
        self._export_btn.setMaximumSize(QSize(125, 35))

        self._import_btn = QPushButton('Import')
        self._import_btn.setObjectName('impotButton')
        self._import_btn.setMinimumSize(QSize(60, 35))
        self._import_btn.setMaximumSize(QSize(125, 35))

        self._reference_btn = QPushButton('Reference')
        self._reference_btn.setObjectName('referenceButton')
        self._reference_btn.setMinimumSize(QSize(60, 35))
        self._reference_btn.setMaximumSize(QSize(125, 35))

        self._preview_buttons_lyt.addWidget(self._export_btn)
        self._preview_buttons_lyt.addWidget(self._import_btn)
        self._preview_buttons_lyt.addWidget(self._reference_btn)

        # By default, export button is disabled
        self._export_btn.setVisible(False)

    def setup_signals(self):
        super(DataPreviewWidget, self).setup_signals()

        self._export_btn.clicked.connect(self.export_data)
        self._import_btn.clicked.connect(self.import_data)
        self._reference_btn.clicked.connect(self.reference_data)

    def export_data(self):
        """
        Export data into hard disk
        """

        if not self.item():
            return

        comment = qtutils.get_comment(parent=self)
        if comment is None:
            return

        self.item().export_data(comment)

        self.update_history()

    def import_data(self):
        """
        Imports data into current scene
        """

        if not self.item():
            return

        self.item().import_data()

    def reference_data(self):
        """
        References data into current scene
        """

        if not self.item():
            return

        self.item().reference_data()


class DataItem(items.BaseItem, object):

    DefaultDataFileName = 'new_data_file'
    PreviewWidgetClass = DataPreviewWidget

    def __init__(self, *args, **kwargs):
        super(DataItem, self).__init__(*args, **kwargs)

        self._data_class = None
        self._data_object = None

    def settings(self):
        """
        Returns tpRigToolkit library settings file
        :return: JSONSettings
        """

        library_settings_root_path = os.getenv('APPDATA') or os.getenv('HOME')
        library_settings_path = os.path.join(library_settings_root_path, 'tpRigToolkit', 'library.json')
        library_settings = settings.JSONSettings(
            directory=os.path.dirname(library_settings_path), filename=os.path.basename(library_settings_path))

        return library_settings

    def write(self, path, objects, icon_path='', sequence_path='', **options):
        """
        Writes all the given object data to the given path on disk
        :param path: str
        :param objects: list(str)
        :param icon_path: str
        :param sequence_path: str
        :param options: dict
        """

        super(DataItem, self).write(
            path=path, objects=objects, icon_path=icon_path, sequence_path=sequence_path, options=options)

        comment = options.get('comment', '-')
        name = options.get('name', self.DefaultDataFileName)
        create_version = options.get('create_version', False)

        data_object = self.data_object(name=name, path=path)
        return data_object.export_data(comment=comment, objects=objects, create_version=create_version)

    def save(self, path=None, *args, **kwargs):
        """
        Saves current item
        :param path:
        :param args:
        :param kwargs:
        :return:
        """

        valid_save = super(DataItem, self).save(path=path, *args, **kwargs)
        if not valid_save:
            return

        # We use the transfer object only to store metadata
        # NOTE: We must do this call here because if not we will try store the file in an non valid path
        # NOTE: because during save function all data is stored in a temporal folder until the creation process
        # is valid
        return self.transfer_object().save(path=self.transfer_path())

    def load(self, objects=None, namespaces=None, **kwargs):
        """
        Loads the data from the transfer object
        :param objects: list(str) or None
        :param namespaces: list(str) or None
        :param kwargs: dict
        """

        stored_path = path_utils.clean_path(os.path.join(self.path(), self.name()))
        data_object_path = path_utils.clean_path(self.data_object().get_file())
        if stored_path != data_object_path:
            self.show_error_dialog(
                'Impossible to open file', 'Stored Path and Data Path are different: {}\n{}'.format(
                    stored_path, data_object_path))
            return

        return self.data_object().open(stored_path)

    def export_data(self, comment):
        """
        Export data from current scene
        :param comment: str
        """

        stored_path = path_utils.clean_path(os.path.join(self.path(), self.name()))
        data_object_path = path_utils.clean_path(self.data_object().get_file())
        if stored_path != data_object_path:
            self.show_error_dialog('Impossible to reference file',
                                   'Stored Path and Data Path are different: {}\n{}'.format(stored_path,
                                                                                            data_object_path))
            return

        return self.data_object().export_data(comment)

    def import_data(self):
        """
        Imports data into current scene
        """

        stored_path = path_utils.clean_path(os.path.join(self.path(), self.name()))
        data_object_path = path_utils.clean_path(self.data_object().get_file())
        if stored_path != data_object_path:
            self.show_error_dialog('Impossible to reference file',
                                   'Stored Path and Data Path are different: {}\n{}'.format(stored_path,
                                                                                            data_object_path))
            return

        return self.data_object().import_data(stored_path)

    def reference_data(self):
        """
        References data into current scene
        """

        stored_path = path_utils.clean_path(os.path.join(self.path(), self.name()))
        data_object_path = path_utils.clean_path(self.data_object().get_file())
        if stored_path != data_object_path:
            self.show_error_dialog('Impossible to import file',
                                   'Stored Path and Data Path are different: {}\n{}'.format(stored_path,
                                                                                            data_object_path))
            return

        return self.data_object().reference_data(stored_path)

    def data_class(self):
        """
        Returns the data class for this item
        :return: Data
        """

        return self._data_class

    def set_data_class(self, class_name):
        """
        Sets the data class for this item
        :param class_name: str
        """

        if hasattr(class_name, 'get_data_extension()'):
            if class_name.Extension != '.{}'.format(class_name.get_data_extension()):
                tpRigToolkit.logger.error(
                    'Data class {} (.{}) has not the same extension as the data item: {} ({})'.format(
                        class_name, class_name.get_data_extension(), self, self.Extension))
                return

        self._data_class = class_name

    def data_object(self, name=None, path=None):
        """
        Returns the data object for this item
        :param name: str
        :param path: str
        :return: Data
        """

        if not self._data_object:
            name = os.path.splitext(name or self.name())[0]
            path = path or self.path()
            self._data_object = self.data_class()(name, path)
            self._data_object.set_directory(path)

        return self._data_object



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains helpers functions to log info during rigging process
"""

from __future__ import print_function, division, absolute_import

import string

import tpDcc as tp
from tpDcc.libs.python import log

import tpRigToolkit


def start_temp_log():
    """
    Initializes a new temp and stores its results in environment variable
    """

    log.start_temp_log(tpRigToolkit.logger.name)


def record_temp_log(value):
    """
    Adds a new value to the temp log with the given name (if exists)
    :param log_name: str, name of the log we want to add value into
    :param value: str
    """

    log.record_temp_log(tpRigToolkit.logger.name, value)


def end_temp_log():
    """
    Removes temp log with given name and returns its contents
    :param log_name: str, nam of the temp log we want to remove
    :return: str
    """

    return log.end_temp_log(tpRigToolkit.logger.name)


def show(*args):
    """
    Helper function that prints given arguments into proper Dcc logs and temp logger
    :param args: list(variant)
    """

    try:
        string_value = show_list_to_string(*args)
        log_value = string_value.replace('\n', '\nLOG:\t\t')
        text = '\t\t{}'.format(string_value)
        print(text)
        log.record_temp_log(tpRigToolkit.logger.name, '\n{}'.format(log_value))
    except RuntimeError as e:
        text = '\t\tCould not show {}'.format(args)
        print(text)
        log.record_temp_log(tpRigToolkit.logger.name, '\n{}'.format(log_value))
        raise RuntimeError(e)


def warning(*args):
    """
    Helper function that prints given arguments as warning into proper Dcc logs and temp logger
    :param args: list(variant)
    """

    try:
        string_value = show_list_to_string(*args)
        log_value = string_value.replace('\n', '\nLOG:\t\t')
        text = '[WARNING]\t{}'.format(string_value)
        if not tp.is_maya():
            print(text)
        else:
            tp.Dcc.warning('LOG: \t{}'.format(string_value))

        log.record_temp_log(tpRigToolkit.logger.name, '\n[WARNING]: {}'.format(log_value))
    except RuntimeError as e:
        raise RuntimeError(e)


def error(*args):
    """
    Helper function that prints given arguments as error into proper Dcc logs and temp logger
    :param args: list(variant)
    """

    try:
        string_value = show_list_to_string(*args)
        log_value = string_value.replace('\n', '\nLOG:\t\t')
        text = '[ERROR]\t{}'.format(string_value)
        print(text)
        log.record_temp_log(tpRigToolkit.logger.name, '\n[ERROR]: {}'.format(log_value))
    except RuntimeError as e:
        raise RuntimeError(e)


def show_list_to_string(*args):
    """
    Converts given arguments into a string
    :param args: list(variant)
    :return: str
    """

    try:
        if args is None:
            return 'None'
        if not args:
            return ''

        new_args = list()
        for arg in args:
            if args is not None:
                new_args.append(str(arg))
        args = new_args
        if not args:
            return ''

        string_value = string.join(args).replace('\n', '\t\n')
        if string_value.endswith('\t\n'):
            string_value = string_value[:-2]

        return string_value
    except RuntimeError as e:
        raise RuntimeError(e)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Skeleton data
"""

from __future__ import print_function, division, absolute_import

import os
import json

import tpDcc as tp
from tpDcc.core import data
from tpDcc.libs.python import fileio

import tpRigToolkit
from tpRigToolkit.core import data as rig_data


class SkeletonFileData(data.CustomData, object):
    def __init__(self, name=None, path=None):
        super(SkeletonFileData, self).__init__(name=name, path=path)

    @staticmethod
    def get_data_type():
        return 'dcc.skeleton'

    @staticmethod
    def get_data_extension():
        return 'skl'

    @staticmethod
    def get_data_title():
        return 'Skeleton'

    def export_data(self, file_path=None, comment='-', create_version=True, *args, **kwargs):

        file_path = file_path or self.get_file()

        objects = kwargs.get('objects', None)
        if not objects:
            objects = tp.Dcc.selected_nodes(full_path=True)
        if not objects:
            tpRigToolkit.logger.warning(
                'Select root node of the skeleton to export or the list of skeleton nodes to export')
            return False

        file_folder = os.path.dirname(file_path)

        root_nodes = list()
        if len(objects) == 1:
            root_nodes.append(objects[0])
        else:
            for object in objects:
                object_parent = tp.Dcc.node_parent(object, full_path=False)
                if not object_parent or not tp.Dcc.node_is_joint(object_parent):
                    root_nodes.append(object_parent)
        if not root_nodes:
            tpRigToolkit.logger.warning('No root joints found!')
            return False
        if len(root_nodes) > 1:
            tpRigToolkit.logger.warning(
                'Multiple root nodes found in skeleton. Only first one will be exported: {}'.format(root_nodes[0]))
        root_node = root_nodes[0]

        skeleton_data = list()
        child_nodes = tp.Dcc.list_children(root_node, children_type='transform')
        child_nodes.insert(0, root_node)

        visited_nodes = dict()
        for i, node in enumerate(child_nodes):
            node_data = dict()
            node_short_name = tp.Dcc.node_short_name(node)
            node_data['name'] = node_short_name
            node_data['index'] = i
            node_data['type'] = tp.Dcc.node_type(node)
            visited_nodes[node_short_name] = i
            world_matrix = tp.Dcc.node_world_matrix(node)
            node_data['world_matrix'] = world_matrix
            parent_index = None
            parent_node = tp.Dcc.node_parent(node)
            if parent_node:
                parent_short_name = tp.Dcc.node_short_name(parent_node)
                if parent_short_name in visited_nodes:
                    parent_index = visited_nodes[parent_short_name]
            if parent_index is None:
                parent_index = -1
            node_data['parent_index'] = parent_index
            skeleton_data.append(node_data)
        if not skeleton_data:
            tpRigToolkit.logger.warning('No skeleton data found!')
            return False
        tpRigToolkit.logger.info('Exporting Skeleton Data: {}'.format(skeleton_data))

        try:
            with open(file_path, 'w') as yaml_file:
                json.dump(skeleton_data, yaml_file, indent=2)
        except IOError:
            tpRigToolkit.logger.error('Skeleton data not saved to file {}'.format(file_path))
            return False

        tpRigToolkit.logger.info('Skeleton data exported successfully!')

        version = fileio.FileVersion(file_folder)
        version.save(comment)

        return True

    def import_data(self, file_path='', objects=None):

        file_path = file_path or self.get_file()
        if not file_path or not os.path.isfile(file_path):
            tpRigToolkit.logger.warning('Impossible to import skeleton data from: "{}"'.format(file_path))
            return False

        with open(file_path, 'r') as fh:
            skeleton_data = json.load(fh)
        if not skeleton_data:
            tpRigToolkit.logger.warning('No skeleton data found in file: "{}"'.format(file_path))
            return False

        created_nodes = dict()
        for node_data in skeleton_data:
            node_index = node_data.get('index', 0)
            node_parent_index = node_data.get('parent_index', -1)
            node_name = node_data.get('name', 'new_node')
            node_type = node_data.get('type', 'joint')
            node_world_matrix = node_data.get(
                'world_matrix', [1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0])
            tp.Dcc.clear_selection()
            if node_type == 'joint':
                new_node = tp.Dcc.create_joint(name=node_name)
            else:
                new_node = tp.Dcc.create_empty_group(name=node_name)
            tp.Dcc.set_node_world_matrix(new_node, node_world_matrix)
            created_nodes[node_index] = {'node': new_node, 'parent_index': node_parent_index}

        for node_index, node_data in created_nodes.items():
            parent_index = node_data['parent_index']
            if parent_index <= -1:
                continue
            node_data = created_nodes.get(node_index, None)
            if not node_data:
                continue
            node_name = node_data.get('node')
            parent_node_data = created_nodes.get(parent_index, None)
            if not parent_node_data:
                continue
            parent_node_name = parent_node_data.get('node')
            tp.Dcc.set_parent(node_name, parent_node_name)


class SkeletonPreviewWidget(rig_data.DataPreviewWidget, object):
    def __init__(self, item, parent=None):
        super(SkeletonPreviewWidget, self).__init__(item=item, parent=parent)


class Skeleton(rig_data.DataItem, object):
    Extension = '.{}'.format(SkeletonFileData.get_data_extension())
    Extensions = ['.{}'.format(SkeletonFileData.get_data_extension())]
    MenuOrder = 5
    MenuName = SkeletonFileData.get_data_title()
    MenuIconName = 'skeleton.png'
    TypeIconName = 'skeleton.png'

    TypeIconName = 'skeleton.png'
    DataType = SkeletonFileData.get_data_type()
    DefaultDataFileName = 'new_skeleton_file'
    PreviewWidgetClass = SkeletonPreviewWidget

    def __init__(self, *args, **kwargs):
        super(Skeleton, self).__init__(*args, **kwargs)

        self.set_data_class(SkeletonFileData)
